;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of AMSTRON: An Amstrad CPC Game 
;;  Copyright (C) 2019 - CPC Games 'La Yema' Francisco Luis Fernández Soriano
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "header.h.s"
.include "entity_manager.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_system_check_by_coord
;;  
;;  
;;  B <- x in Screen
;;  C <- y in Screen
;;
;;  OUTPUT A >> content hl
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

collision_system_check_by_coord::
    ld hl, #_matrix
    call input_get_position_in_matrix 
    call collision_check_in_matrix
    ld a, ( hl )
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_system_init_matrix
;;  
;;  
;;  C -> X Coord. X Matrix
;;  D -> Y Coord. Y Matrix 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

collision_system_init_matrix_by_arena::
	cp #ARENA_1
	ret z
	cp #ARENA_2
	jr z, insert_into_matrix_coord_arena_2
	cp #ARENA_3
	jr z, insert_into_matrix_coord_arena_3
	cp #ARENA_4
	ret z
	cp #ARENA_5
	jr z, insert_into_matrix_coord_arena_2
	cp #ARENA_6
	jr z, insert_into_matrix_coord_arena_3
	cp #ARENA_7
	ret z
	cp #ARENA_8
	jr z, insert_into_matrix_coord_arena_4
	cp #ARENA_9
	jr z, insert_into_matrix_coord_arena_3

insert_into_matrix_coord_arena_2:
	ld b, #0x08
	ld c, #0x07
loop_arena2_y:
	inc c
	ld b, #0x08
	ld a,c 
	cp #0x0C
	ret z

loop_arena2_x:

	push bc
	ld hl, #_matrix
	call collision_get_hl_in_matrix_by_x_y
	pop bc
	ld (hl), #WALL
	inc b
	ld a, b
	cp #0x0C
	jr z, loop_arena2_y
	jr loop_arena2_x
ret

insert_into_matrix_coord_arena_3:
	ld c, #0x0B
loop_arena3_y:
	inc c
	ld b, #0x03
	ld a,c 
	cp #0x11
	jr z, arena3_second_square

loop_arena3_x:
	push bc
	ld hl, #_matrix
	call collision_get_hl_in_matrix_by_x_y
	pop bc
	ld (hl), #WALL
	inc b
	ld a, b
	cp #0x07
	jr z, loop_arena3_y
	jr loop_arena3_x

arena3_second_square:
	ld c, #0x02
loop_arena3_y_second_square:
	inc c
	ld b, #0x0E
	ld a,c 
	cp #0x08
	ret z
loop_arena3_x_second_square:
	push bc
	ld hl, #_matrix
	call collision_get_hl_in_matrix_by_x_y
	pop bc
	ld (hl), #WALL
	inc b
	ld a, b
	cp #0x11
	jr z, loop_arena3_y_second_square
	jr loop_arena3_x_second_square


ret

insert_into_matrix_coord_arena_4:
	call collision_clean_all_matrix
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_update_score
;;  
;;  
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

collision_update_score:
	ld a, #0x0A
	ld b, e_lives(ix)
	sub b 
	call render_score_update



ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_calculate_tileset_by_int_tile
;;  
;;  
;;  a <- Tile Int
;;  
;;
;;  HL --> Tile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


collision_calculate_tileset_by_int_tile::
	cp #0x01
	jp z, tileset_1
	cp #0x06
	jp z, tileset_6
	cp #0x07
	jp z, tileset_7
	cp #0x04
	jp z, tileset_4
	cp #0x08
	jp z, tileset_8
	cp #0x09
	jp z, tileset_9
	cp #10
	jp z, tileset_10
	cp #11
	jp z, tileset_11
	cp #12
	jp z, tileset_12
	cp #13
	jp z, tileset_13
	cp #14
	jp z, tileset_14
	cp #15
	jp z, tileset_15
	cp #16
	jp z, tileset_16
	cp #17
	jp z, tileset_17
	cp #18
	jp z, tileset_18
	cp #19
	jp z, tileset_19
	cp #20
	jp z, tileset_20
	cp #21
	jp z, tileset_21
	cp #22
	jp z, tileset_22
	cp #23
	jp z, tileset_23
	cp #24
	jp z, tileset_24
	cp #25
	jp z, tileset_25
	cp #26
	jp z, tileset_26
	cp #27
	jp z, tileset_27
	cp #28
	jp z, tileset_28
	cp #29
	jp z, tileset_29
	cp #30
	jp z, tileset_30
	cp #31
	jp z, tileset_31
	cp #32
	jp z, tileset_32
	cp #33
	jp z, tileset_33
	cp #34
	jp z, tileset_34
	cp #35
	jp z, tileset_35
	cp #36
	jp z, tileset_36
	cp #37
	jp z, tileset_37
	cp #38
	jp z, tileset_38
	cp #39
	jp z, tileset_39
	cp #40
	jp z, tileset_40
	cp #41
	jp z, tileset_41
	cp #42
	jp z, tileset_42
	cp #43
	jp z, tileset_43
	cp #44
	jp z, tileset_44
	cp #45
	jp z, tileset_45
	cp #46
	jp z, tileset_46
	

tileset_1: 
	ld hl, #_g_tileset_01
ret
tileset_6: 
	ld hl, #_g_tileset_06
ret
tileset_7: 
	ld hl, #_g_tileset_07
ret
tileset_4: 
	ld hl, #_g_tileset_04
ret
tileset_8: 
	ld hl, #_g_tileset_08
ret
tileset_9:
	ld hl, #_g_tileset_09
ret
tileset_10: 
	ld hl, #_g_tileset_10
ret
tileset_11: 
	ld hl, #_g_tileset_11
ret
tileset_12: 
	ld hl, #_g_tileset_12
ret
tileset_13: 
	ld hl, #_g_tileset_13
ret
tileset_14: 
	ld hl, #_g_tileset_14
ret
tileset_15: 
	ld hl, #_g_tileset_15
ret
tileset_16: 
	ld hl, #_g_tileset_16
ret
tileset_17: 
	ld hl, #_g_tileset_17
ret
tileset_18: 
	ld hl, #_g_tileset_18
ret
tileset_19: 
	ld hl, #_g_tileset_19
ret
tileset_20: 
	ld hl, #_g_tileset_20
ret
tileset_21: 
	ld hl, #_g_tileset_21
ret
tileset_22: 
	ld hl, #_g_tileset_22
ret
tileset_23: 
	ld hl, #_g_tileset_23
ret
tileset_24: 
	ld hl, #_g_tileset_24
ret
tileset_25: 
	ld hl, #_g_tileset_25
ret
tileset_26: 
	ld hl, #_g_tileset_26
ret
tileset_27: 
	ld hl, #_g_tileset_27
ret
tileset_28: 
	ld hl, #_g_tileset_28
ret
tileset_29: 
	ld hl, #_g_tileset_29
ret
tileset_30: 
	ld hl, #_g_tileset_30
ret
tileset_31: 
	ld hl, #_g_tileset_31
ret
tileset_32: 
	ld hl, #_g_tileset_32
ret
tileset_33: 
	ld hl, #_g_tileset_33
ret
tileset_34: 
	ld hl, #_g_tileset_34
ret
tileset_35: 
	ld hl, #_g_tileset_35
ret
tileset_36: 
	ld hl, #_g_tileset_36
ret
tileset_37: 
	ld hl, #_g_tileset_37
ret
tileset_38: 
	ld hl, #_g_tileset_38
ret
tileset_39: 
	ld hl, #_g_tileset_39
ret
tileset_40: 
	ld hl, #_g_tileset_40
ret
tileset_41: 
	ld hl, #_g_tileset_41
ret
tileset_42: 
	ld hl, #_g_tileset_42
ret
tileset_43: 
	ld hl, #_g_tileset_43
ret
tileset_44: 
	ld hl, #_g_tileset_44
ret
tileset_45: 
	ld hl, #_g_tileset_45
ret
tileset_46: 
	ld hl, #_g_tileset_46
ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_calculate_level_by_arena
;;  
;;  
;;  
;;  HL -> Level 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

collision_calculate_level::
	ld a, ( #_arena )
	cp #ARENA_1
	jr z, assign_level1
	cp #ARENA_2
	jr z, assign_level2
	cp #ARENA_3
	jr z, assign_level3
	cp #ARENA_4
	jr z, assign_level4
	cp #ARENA_5
	jr z, assign_level5
	cp #ARENA_6
	jr z, assign_level5
	cp #ARENA_7
	jr z, assign_level1
	cp #ARENA_8
	jr z, assign_level4
	cp #ARENA_9
	jr z, assign_level6
assign_level1:	
	ld hl, #_level1
	jr continue_collision_calculate_tile
assign_level2:	
	ld hl, #_level2
	jr continue_collision_calculate_tile
assign_level3:	
	ld hl, #_level3
	jr continue_collision_calculate_tile
assign_level4:	
	ld hl, #_level4
	jr continue_collision_calculate_tile
assign_level5:	
	ld hl, #_level4
	jr continue_collision_calculate_tile
assign_level6:	
	ld hl, #_level4
	
continue_collision_calculate_tile:

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_calculate_tile
;;  
;;  
;;  C -> X Coord. X Matrix
;;  D -> Y Coord. Y Matrix 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

collision_calculate_tile::
	
	call collision_calculate_level
	ld a, d
loop_level_y:
	cp #0x00
	jr z, loop_level_x
	dec a
	push de
	ld de, #0x0014
	add hl, de
	pop de
	jr loop_level_y

loop_level_x:
	ld d, #0x00
	ld e, c
	add hl,de
	ld a, (hl)
	call collision_calculate_tileset_by_int_tile
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_clean_matrix
;;  
;;  INPUT HL -> Posición de memoria
;;  C -> X Coord. X Matrix
;;  D -> Y Coord. Y Matrix 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

collision_draw_tile:
	push hl
      push de
      push bc
      ;;;;;;;;;;;;;;;;;;;;;;
      call collision_calculate_tile
      pop bc
      pop de
      push de
      push bc
      push hl
      sla c
      sla c
      sla d
      sla d
      sla d
      ld a, d
      add #0x28
      ld d, a 
  	ld b, d
  	ld de, #0xC000
  	call cpct_getScreenPtr_asm

      ex de, hl
      pop hl
      call cpct_drawTileGrayCode4x8_af_asm

      ;;;;;;;;;;;;;;;;;;;;;
 	pop bc
      pop de
      pop hl
      ld (hl), #0xFF
ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_check_in_matrix
;;  
;;  INPUT B --> Collision Matrix 
;;  INPUT C --> Collision Matrix
;;  OUTPUT -D -> Collision 1 or 0 
;;  OUTPUT E --> HL CONTENT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
collision_check_in_matrix::
    ld a, (hl)
    ld e, a
    ld d, #MOTORBIKE_NO_COLLISION
    cp #0xFF
    jr z, no_collision
    ld a, e_changedir(ix)
    cp #1
    jr z, change_direction
    ld d, #MOTORBIKE_COLLISION
change_direction:
	ld a, e_x(ix)
	ld b, e_ma_x(ix)
	cp b
	jr z, check_x_near
	ld d, #MOTORBIKE_COLLISION
check_x_near:
	dec l
	ld a, (hl)
	cp #0xFF
	jr z, no_collision
	inc l
	inc l
	ld a, (hl)
	cp #0xFF
	jr z, no_collision
	ld d, #MOTORBIKE_COLLISION
no_collision: 
	
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_get_hl_in_matrix_by_x_y
;;  
;;  
;;   
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
collision_get_hl_in_matrix_by_x_y:
	
y_matrix_by_x_y:
	ld a, c
	cp #0x00
	jr z, y_matrix_by_x_x
	dec c
	ld de, #0x0014
	add hl, de
	jr y_matrix_by_x_y

y_matrix_by_x_x:
	ld d, #0x00
	ld e, b
	add hl,de

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_system_clean_motorbike_collision
;;  
;;  
;;   
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
collision_system_clean_motorbike_collision:

	call collision_calculate_level
	ld b, e_ma_x(ix)
	ld c, e_ma_y(ix)
	call input_get_position_in_matrix
	ld a, ( hl )
	call collision_calculate_tileset_by_int_tile
      push hl
      ld de, #0xC000
	ld c, e_ma_x(ix)
	ld b, e_ma_y(ix)
  	call cpct_getScreenPtr_asm
      ex de, hl
      pop hl
      call cpct_drawTileGrayCode4x8_af_asm	
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_clean_all_matrix
;;  
;;  
;;   
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
collision_clean_all_matrix::
	ld hl, #_matrix
	ld c, #0x00
	ld d, #0x00
loop_all_matrix:
	ld (hl), #0xFF
	inc hl
	inc c
	ld a,c
	cp #0x14
	jr z, reset_y_all_matrix
	jr loop_all_matrix
reset_y_all_matrix:
	ld c, #0x00
	inc d
	ld a,d
	cp #0x14
	ret z
	jr loop_all_matrix

ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  collision_clean_matrix
;;  
;;  INPUT B -> Player 1 or Player 2 or Robot
;;   
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
collision_clean_matrix_by_player::
	ld hl, #_matrix
	ld c, #0x00
	ld d, #0x00
	jr player_collision_draw_tile
loop_matrix:
	ld a, (hl)
	cp #0xFF
	jr z, player_collision_draw_tile
	cp b
	jr z, player_collision_draw_tile
	cp #WALL
	jr z, player_collision_draw_tile
begin_loop_matrix:
	inc hl
	inc c
	ld a,c
	cp #0x14
	jr z, reset_y
	jr loop_matrix
reset_y:
	ld c, #0x00
	inc d
	ld a,d
	cp #0x14
	jr z, exit_collision_clean_matrix
	jr loop_matrix

player_collision_draw_tile:
	
	call collision_draw_tile 
	jr begin_loop_matrix
exit_collision_clean_matrix:
ret

colision_is_true_in_matrix:
	ld a, #0x01
	ld e_collision(ix), a
	ld a, e_dir(ix)
	cp #MOTORBIKE_DIR_UP
	jr z, collision_up_is_true
	cp #MOTORBIKE_DIR_LEFT
	jr z, collision_left_is_true
	jr end_colision_is_true
collision_left_is_true:
	call render_system_erase_entity
	ld a, e_ma_x(ix)
    	;;sub #0x02
    	ld e_ma_x(ix), a
    	ld a, #DESTROY_ANIMATION_HORINZONTAL_ONE
    	ld e_anim(ix),a
    	ld a, #0x04
    	ld e_w_ant(ix), a
    	ld a, #0x04
    	ld e_h_ant(ix), a
    	jr end_colision_is_true
collision_up_is_true:
	call render_system_erase_entity
	ld a, e_ma_y(ix)
    	sub #0x08
    	ld e_ma_y(ix), a
    	ld a, #DESTROY_ANIMATION_VERTICAL_ONE
    	ld e_anim(ix),a
    	ld a, #0x01
    	ld e_w_ant(ix), a
    	ld a, #0x10
    	ld e_h_ant(ix), a
    	
end_colision_is_true:
	jp not_collision

colision_is_true:
	ld a, #0x01
	ld e_collision(ix), a
	jp not_collision

collision_system_update::

	call entityman_getEntityArray_IX
 	call entityman_getNumEntities_A

_renloop_collision:
	push af

	ld a, e_player(ix)
	cp #ROBOT
	jp z, check_collision_robot
	jr continue_collision_system_update

check_collision_robot:
	ld a, ( #_red_robot )
	cp #0x00
	jp z, not_collision
	ld b, e_x(ix)
	ld c, e_y(ix)
	ld hl, #_matrix
	call input_store_mov_in_matrix
      jp not_collision

continue_collision_system_update:
	ld a, e_collision(ix)
    	cp #0x01
    	jr z, check_end_collision

	ld a, e_x(ix)
	cp #LIMIT_X_RIGHT
	jp z, colision_is_true
	cp #LIMIT_X_LEFT
	jp z, colision_is_true
      ld a, e_y(ix)
	cp #LIMIT_Y_UP
	jp z, colision_is_true
	cp #LIMIT_Y_DOWN
	jp z, colision_is_true

	ld b, e_x(ix)
	ld c, e_y(ix)
	ld hl, #_matrix
	call input_get_position_in_matrix 
	call collision_check_in_matrix
	ld a, d
	cp #MOTORBIKE_COLLISION
	jp z, colision_is_true_in_matrix
	ld b, e_x(ix)
	ld c, e_y(ix)
	ld hl, #_matrix
	call input_store_mov_in_matrix
      jp not_collision
	



check_end_collision:
    ld a, e_anim(ix)
    cp #DESTROY_ANIMATION_HORINZONTAL_THREE
    jr z, reset_collision
    cp #DESTROY_ANIMATION_VERTICAL_THREE
    jr z, reset_collision
    ld a, e_anim(ix)
    add #0x02
    ld e_anim(ix),a
    jr not_collision

reset_collision:
   ;;call collision_system_clean_motorbike_collision	
   ld a,e_lives(ix)
   dec a
   ld e_lives(ix),a 
   call collision_update_score
   ld a,e_lives(ix)
   cp #NUM_LIVES
   jr z, reset_input_pause
   ld b, e_player(ix)
   call collision_clean_matrix_by_player
   ld a, ( #_arena )
   call collision_system_init_matrix_by_arena

reset_continue:
    ld e_anim(ix), #0x00
    ld e_reset(ix), #RESET_PLAYER
    ld e_collision(ix), #0x00    
    ld a, e_reset(ix)
    cp #RESET_PLAYER
    jr z, collision_input_entity_reset
    jp not_collision
collision_input_entity_reset:
	call input_entity_reset


not_collision:

    pop af
    dec a
    ret z ;; Equals to JZ  
    ld bc, #sizeof_e
    add ix, bc
    jp _renloop_collision
reset_input_pause: 
 	 pop af
 	 ld a, #PAUSE
 	 ld ( #_pause_level), a
	 call collision_clean_all_matrix
ret
